@@ -3,7 +3,7 @@
  * regexp.c
  *	  Postgres' interface to the regular expression package.
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -423,8 +423,8 @@ parse_re_flags(pg_re_flags *flags, text *opts)
 				default:
 					ereport(ERROR,
 							(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-							 errmsg("invalid regular expression option: \"%c\"",
-									opt_p[i])));
+							 errmsg("invalid regular expression option: \"%.*s\"",
+									pg_mblen(opt_p + i), opt_p + i)));
 					break;
 			}
 		}
@@ -673,9 +673,8 @@ similar_escape_internal(text *pat_text, text *esc_text)
 	int			plen,
 				elen;
 	bool		afterescape = false;
+	bool		incharclass = false;
 	int			nquotes = 0;
-	int			bracket_depth = 0;	/* square bracket nesting level */
-	int			charclass_pos = 0;	/* position inside a character class */
 
 	p = VARDATA_ANY(pat_text);
 	plen = VARSIZE_ANY_EXHDR(pat_text);
@@ -734,17 +733,6 @@ similar_escape_internal(text *pat_text, text *esc_text)
 	 * the relevant part separators in the above expansion.  If the result
 	 * of this function is used in a plain regexp match (SIMILAR TO), the
 	 * escape-double-quotes have no effect on the match behavior.
-	 *
-	 * While we don't fully validate character classes (bracket expressions),
-	 * we do need to parse them well enough to know where they end.
-	 * "charclass_pos" tracks where we are in a character class.
-	 * Its value is uninteresting when bracket_depth is 0.
-	 * But when bracket_depth > 0, it will be
-	 *   1: right after the opening '[' (a following '^' will negate
-	 *      the class, while ']' is a literal character)
-	 *   2: right after a '^' after the opening '[' (']' is still a literal
-	 *      character)
-	 *   3 or more: further inside the character class (']' ends the class)
 	 *----------
 	 */
 
@@ -816,7 +804,7 @@ similar_escape_internal(text *pat_text, text *esc_text)
 		/* fast path */
 		if (afterescape)
 		{
-			if (pchar == '"' && bracket_depth < 1)	/* escape-double-quote? */
+			if (pchar == '"' && !incharclass)	/* escape-double-quote? */
 			{
 				/* emit appropriate part separator, per notes above */
 				if (nquotes == 0)
@@ -857,12 +845,6 @@ similar_escape_internal(text *pat_text, text *esc_text)
 				 */
 				*r++ = '\\';
 				*r++ = pchar;
-
-				/*
-				 * If we encounter an escaped character in a character class,
-				 * we are no longer at the beginning.
-				 */
-				charclass_pos = 3;
 			}
 			afterescape = false;
 		}
@@ -871,69 +853,18 @@ similar_escape_internal(text *pat_text, text *esc_text)
 			/* SQL escape character; do not send to output */
 			afterescape = true;
 		}
-		else if (bracket_depth > 0)
+		else if (incharclass)
 		{
-			/* inside a character class */
 			if (pchar == '\\')
-			{
-				/*
-				 * If we're here, backslash is not the SQL escape character,
-				 * so treat it as a literal class element, which requires
-				 * doubling it.  (This matches our behavior for backslashes
-				 * outside character classes.)
-				 */
 				*r++ = '\\';
-			}
 			*r++ = pchar;
-
-			/* parse the character class well enough to identify ending ']' */
-			if (pchar == ']' && charclass_pos > 2)
-			{
-				/* found the real end of a bracket pair */
-				bracket_depth--;
-				/* don't reset charclass_pos, this may be an inner bracket */
-			}
-			else if (pchar == '[')
-			{
-				/* start of a nested bracket pair */
-				bracket_depth++;
-
-				/*
-				 * We are no longer at the beginning of a character class.
-				 * (The nested bracket pair is a collating element, not a
-				 * character class in its own right.)
-				 */
-				charclass_pos = 3;
-			}
-			else if (pchar == '^')
-			{
-				/*
-				 * A caret right after the opening bracket negates the
-				 * character class.  In that case, the following will
-				 * increment charclass_pos from 1 to 2, so that a following
-				 * ']' is still a literal character and does not end the
-				 * character class.  If we are further inside a character
-				 * class, charclass_pos might get incremented past 3, which is
-				 * fine.
-				 */
-				charclass_pos++;
-			}
-			else
-			{
-				/*
-				 * Anything else (including a backslash or leading ']') is an
-				 * element of the character class, so we are no longer at the
-				 * beginning of the class.
-				 */
-				charclass_pos = 3;
-			}
+			if (pchar == ']')
+				incharclass = false;
 		}
 		else if (pchar == '[')
 		{
-			/* start of a character class */
 			*r++ = pchar;
-			bracket_depth = 1;
-			charclass_pos = 1;
+			incharclass = true;
 		}
 		else if (pchar == '%')
 		{
