@@ -42,7 +42,7 @@
  * where memory fragmentation is very severe, only a tiny fraction of
  * the pages under management are consumed by this btree.
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2021, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -434,7 +434,7 @@ FreePageManagerDump(FreePageManager *fpm)
 
 	/* Dump general stuff. */
 	appendStringInfo(&buf, "metadata: self %zu max contiguous pages = %zu\n",
-					 fpm->self.relptr_off, fpm->contiguous_pages);
+					 relptr_offset(fpm->self), fpm->contiguous_pages);
 
 	/* Dump btree. */
 	if (fpm->btree_depth > 0)
@@ -894,14 +894,14 @@ FreePageBtreeGetRecycled(FreePageManager *fpm)
 }
 
 /*
- * Insert an item into an internal page (there must be room).
+ * Insert an item into an internal page.
  */
 static void
 FreePageBtreeInsertInternal(char *base, FreePageBtree *btp, Size index,
 							Size first_page, FreePageBtree *child)
 {
 	Assert(btp->hdr.magic == FREE_PAGE_INTERNAL_MAGIC);
-	Assert(btp->hdr.nused < FPM_ITEMS_PER_INTERNAL_PAGE);
+	Assert(btp->hdr.nused <= FPM_ITEMS_PER_INTERNAL_PAGE);
 	Assert(index <= btp->hdr.nused);
 	memmove(&btp->u.internal_key[index + 1], &btp->u.internal_key[index],
 			sizeof(FreePageBtreeInternalKey) * (btp->hdr.nused - index));
@@ -911,14 +911,14 @@ FreePageBtreeInsertInternal(char *base, FreePageBtree *btp, Size index,
 }
 
 /*
- * Insert an item into a leaf page (there must be room).
+ * Insert an item into a leaf page.
  */
 static void
 FreePageBtreeInsertLeaf(FreePageBtree *btp, Size index, Size first_page,
 						Size npages)
 {
 	Assert(btp->hdr.magic == FREE_PAGE_LEAF_MAGIC);
-	Assert(btp->hdr.nused < FPM_ITEMS_PER_LEAF_PAGE);
+	Assert(btp->hdr.nused <= FPM_ITEMS_PER_LEAF_PAGE);
 	Assert(index <= btp->hdr.nused);
 	memmove(&btp->u.leaf_key[index + 1], &btp->u.leaf_key[index],
 			sizeof(FreePageBtreeLeafKey) * (btp->hdr.nused - index));
@@ -1269,7 +1269,7 @@ FreePageManagerDumpBtree(FreePageManager *fpm, FreePageBtree *btp,
 		if (btp->hdr.magic == FREE_PAGE_INTERNAL_MAGIC)
 			appendStringInfo(buf, " %zu->%zu",
 							 btp->u.internal_key[index].first_page,
-							 btp->u.internal_key[index].child.relptr_off / FPM_PAGE_SIZE);
+							 relptr_offset(btp->u.internal_key[index].child) / FPM_PAGE_SIZE);
 		else
 			appendStringInfo(buf, " %zu(%zu)",
 							 btp->u.leaf_key[index].first_page,
@@ -1859,7 +1859,7 @@ FreePagePopSpanLeader(FreePageManager *fpm, Size pageno)
 	{
 		Size		f = Min(span->npages, FPM_NUM_FREELISTS) - 1;
 
-		Assert(fpm->freelist[f].relptr_off == pageno * FPM_PAGE_SIZE);
+		Assert(relptr_offset(fpm->freelist[f]) == pageno * FPM_PAGE_SIZE);
 		relptr_copy(fpm->freelist[f], span->next);
 	}
 }
