@@ -453,7 +453,7 @@ create table trigtest2 (i int references trigtest(i) on delete cascade);
 
 create function trigtest() returns trigger as $$
 begin
-	raise notice '% % % %', TG_RELNAME, TG_OP, TG_WHEN, TG_LEVEL;
+	raise notice '% % % %', TG_TABLE_NAME, TG_OP, TG_WHEN, TG_LEVEL;
 	return new;
 end;$$ language plpgsql;
 
@@ -689,7 +689,7 @@ begin
         argstr := argstr || TG_argv[i];
     end loop;
 
-    raise notice '% % % % (%)', TG_RELNAME, TG_WHEN, TG_OP, TG_LEVEL, argstr;
+    raise notice '% % % % (%)', TG_TABLE_NAME, TG_WHEN, TG_OP, TG_LEVEL, argstr;
 
     if TG_LEVEL = 'ROW' then
         if TG_OP = 'INSERT' then
@@ -1848,27 +1848,6 @@ alter table parent enable always trigger tg;
 select tgrelid::regclass, tgname, tgenabled from pg_trigger
   where tgrelid in ('parent'::regclass, 'child1'::regclass)
   order by tgrelid::regclass::text, tgname;
--- This variant malfunctioned in some releases.
-alter table parent disable trigger user;
-select tgrelid::regclass, tgname, tgenabled from pg_trigger
-  where tgrelid in ('parent'::regclass, 'child1'::regclass)
-  order by tgrelid::regclass::text, tgname;
-drop table parent, child1;
-
--- Check processing of foreign key triggers
-create table parent (a int primary key, f int references parent)
-  partition by list (a);
-create table child1 partition of parent for values in (1);
-select tgrelid::regclass, rtrim(tgname, '0123456789') as tgname,
-  tgfoid::regproc, tgenabled
-  from pg_trigger where tgrelid in ('parent'::regclass, 'child1'::regclass)
-  order by tgrelid::regclass::text, tgfoid;
--- Before v15, this has no effect because parent has no triggers:
-alter table parent disable trigger all;
-select tgrelid::regclass, rtrim(tgname, '0123456789') as tgname,
-  tgfoid::regproc, tgenabled
-  from pg_trigger where tgrelid in ('parent'::regclass, 'child1'::regclass)
-  order by tgrelid::regclass::text, tgfoid;
 drop table parent, child1;
 
 -- Verify that firing state propagates correctly on creation, too
@@ -2038,11 +2017,6 @@ BBB	42
 CCC	42
 \.
 
--- check detach/reattach behavior; statement triggers with transition tables
--- should not prevent a table from becoming a partition again
-alter table parent detach partition child1;
-alter table parent attach partition child1 for values in ('AAA');
-
 -- DML affecting parent sees tuples collected from children even if
 -- there is no transition table trigger on the children
 drop trigger child1_insert_trig on child1;
@@ -2120,52 +2094,6 @@ alter table parent attach partition child for values in ('AAA');
 
 drop table child, parent;
 
---
--- Verify access of transition tables with UPDATE triggers and tuples
--- moved across partitions.
---
-create or replace function dump_update_new() returns trigger language plpgsql as
-$$
-  begin
-    raise notice 'trigger = %, new table = %', TG_NAME,
-                 (select string_agg(new_table::text, ', ' order by a) from new_table);
-    return null;
-  end;
-$$;
-create or replace function dump_update_old() returns trigger language plpgsql as
-$$
-  begin
-    raise notice 'trigger = %, old table = %', TG_NAME,
-                 (select string_agg(old_table::text, ', ' order by a) from old_table);
-    return null;
-  end;
-$$;
-create table trans_tab_parent (a text) partition by list (a);
-create table trans_tab_child1 partition of trans_tab_parent for values in ('AAA1', 'AAA2');
-create table trans_tab_child2 partition of trans_tab_parent for values in ('BBB1', 'BBB2');
-create trigger trans_tab_parent_update_trig
-  after update on trans_tab_parent referencing old table as old_table
-  for each statement execute procedure dump_update_old();
-create trigger trans_tab_parent_insert_trig
-  after insert on trans_tab_parent referencing new table as new_table
-  for each statement execute procedure dump_insert();
-create trigger trans_tab_parent_delete_trig
-  after delete on trans_tab_parent referencing old table as old_table
-  for each statement execute procedure dump_delete();
-insert into trans_tab_parent values ('AAA1'), ('BBB1');
--- should not trigger access to new table when moving across partitions.
-update trans_tab_parent set a = 'BBB2' where a = 'AAA1';
-drop trigger trans_tab_parent_update_trig on trans_tab_parent;
-create trigger trans_tab_parent_update_trig
-  after update on trans_tab_parent referencing new table as new_table
-  for each statement execute procedure dump_update_new();
--- should not trigger access to old table when moving across partitions.
-update trans_tab_parent set a = 'AAA2' where a = 'BBB1';
-delete from trans_tab_parent;
--- clean up
-drop table trans_tab_parent, trans_tab_child1, trans_tab_child2;
-drop function dump_update_new, dump_update_old;
-
 --
 -- Verify behavior of statement triggers on (non-partition)
 -- inheritance hierarchy with transition tables; similar to the
@@ -2256,17 +2184,12 @@ CCC	42
 \.
 
 -- same behavior for copy if there is an index (interesting because rows are
--- captured by a different code path in copy.c if there are indexes)
+-- captured by a different code path in copyfrom.c if there are indexes)
 create index on parent(b);
 copy parent (a, b) from stdin;
 DDD	42
 \.
 
--- check disinherit/reinherit behavior; statement triggers with transition
--- tables should not prevent a table from becoming an inheritance child again
-alter table child1 no inherit parent;
-alter table child1 inherit parent;
-
 -- DML affecting parent sees tuples collected from children even if
 -- there is no transition table trigger on the children
 drop trigger child1_insert_trig on child1;
@@ -2462,20 +2385,6 @@ select * from trig_table;
 
 drop table refd_table, trig_table;
 
---
--- Test that we can drop a not-yet-fired deferred trigger
---
-
-create table refd_table (id int primary key);
-create table trig_table (fk int references refd_table initially deferred);
-
-begin;
-insert into trig_table values (1);
-drop table refd_table cascade;
-commit;
-
-drop table trig_table;
-
 --
 -- self-referential FKs are even more fun
 --
@@ -2511,6 +2420,93 @@ drop function dump_insert();
 drop function dump_update();
 drop function dump_delete();
 
+--
+-- Tests for CREATE OR REPLACE TRIGGER
+--
+create table my_table (id integer);
+
+create function funcA() returns trigger as $$
+begin
+  raise notice 'hello from funcA';
+  return null;
+end; $$ language plpgsql;
+
+create function funcB() returns trigger as $$
+begin
+  raise notice 'hello from funcB';
+  return null;
+end; $$ language plpgsql;
+
+create trigger my_trig
+  after insert on my_table
+  for each row execute procedure funcA();
+
+create trigger my_trig
+  before insert on my_table
+  for each row execute procedure funcB();  -- should fail
+
+insert into my_table values (1);
+
+create or replace trigger my_trig
+  before insert on my_table
+  for each row execute procedure funcB();  -- OK
+
+insert into my_table values (2);  -- this insert should become a no-op
+
+table my_table;
+
+drop table my_table;
+
+-- test CREATE OR REPLACE TRIGGER on partition table
+create table parted_trig (a int) partition by range (a);
+create table parted_trig_1 partition of parted_trig
+       for values from (0) to (1000) partition by range (a);
+create table parted_trig_1_1 partition of parted_trig_1 for values from (0) to (100);
+create table parted_trig_2 partition of parted_trig for values from (1000) to (2000);
+create table default_parted_trig partition of parted_trig default;
+
+-- test that trigger can be replaced by another one
+-- at the same level of partition table
+create or replace trigger my_trig
+  after insert on parted_trig
+  for each row execute procedure funcA();
+insert into parted_trig (a) values (50);
+create or replace trigger my_trig
+  after insert on parted_trig
+  for each row execute procedure funcB();
+insert into parted_trig (a) values (50);
+
+-- test that child trigger cannot be replaced directly
+create or replace trigger my_trig
+  after insert on parted_trig
+  for each row execute procedure funcA();
+insert into parted_trig (a) values (50);
+create or replace trigger my_trig
+  after insert on parted_trig_1
+  for each row execute procedure funcB();  -- should fail
+insert into parted_trig (a) values (50);
+drop trigger my_trig on parted_trig;
+insert into parted_trig (a) values (50);
+
+-- test that user trigger can be overwritten by one defined at upper level
+create trigger my_trig
+  after insert on parted_trig_1
+  for each row execute procedure funcA();
+insert into parted_trig (a) values (50);
+create trigger my_trig
+  after insert on parted_trig
+  for each row execute procedure funcB();  -- should fail
+insert into parted_trig (a) values (50);
+create or replace trigger my_trig
+  after insert on parted_trig
+  for each row execute procedure funcB();
+insert into parted_trig (a) values (50);
+
+-- cleanup
+drop table parted_trig;
+drop function funcA();
+drop function funcB();
+
 -- Leave around some objects for other tests
 create table trigger_parted (a int primary key) partition by list (a);
 create function trigger_parted_trigfunc() returns trigger language plpgsql as
@@ -2539,7 +2535,7 @@ insert into convslot_test_child(col1) values ('1');
 insert into convslot_test_parent(col1) values ('3');
 insert into convslot_test_child(col1) values ('3');
 
-create function convslot_trig1()
+create or replace function trigger_function1()
 returns trigger
 language plpgsql
 AS $$
@@ -2550,7 +2546,7 @@ raise notice 'trigger = %, old_table = %',
 return null;
 end; $$;
 
-create function convslot_trig2()
+create or replace function trigger_function2()
 returns trigger
 language plpgsql
 AS $$
@@ -2563,11 +2559,11 @@ end; $$;
 
 create trigger but_trigger after update on convslot_test_child
 referencing new table as new_table
-for each statement execute function convslot_trig2();
+for each statement execute function trigger_function2();
 
 update convslot_test_parent set col1 = col1 || '1';
 
-create function convslot_trig3()
+create or replace function trigger_function3()
 returns trigger
 language plpgsql
 AS $$
@@ -2581,45 +2577,12 @@ end; $$;
 
 create trigger but_trigger2 after update on convslot_test_child
 referencing old table as old_table new table as new_table
-for each statement execute function convslot_trig3();
+for each statement execute function trigger_function3();
 update convslot_test_parent set col1 = col1 || '1';
 
 create trigger bdt_trigger after delete on convslot_test_child
 referencing old table as old_table
-for each statement execute function convslot_trig1();
+for each statement execute function trigger_function1();
 delete from convslot_test_parent;
 
 drop table convslot_test_child, convslot_test_parent;
-drop function convslot_trig1();
-drop function convslot_trig2();
-drop function convslot_trig3();
-
--- Bug #17607: variant of above in which trigger function raises an error;
--- we don't see any ill effects unless trigger tuple requires mapping
-
-create table convslot_test_parent (id int primary key, val int)
-partition by range (id);
-
-create table convslot_test_part (val int, id int not null);
-
-alter table convslot_test_parent
-  attach partition convslot_test_part for values from (1) to (1000);
-
-create function convslot_trig4() returns trigger as
-$$begin raise exception 'BOOM!'; end$$ language plpgsql;
-
-create trigger convslot_test_parent_update
-    after update on convslot_test_parent
-    referencing old table as old_rows new table as new_rows
-    for each statement execute procedure convslot_trig4();
-
-insert into convslot_test_parent (id, val) values (1, 2);
-
-begin;
-savepoint svp;
-update convslot_test_parent set val = 3;  -- error expected
-rollback to savepoint svp;
-rollback;
-
-drop table convslot_test_parent;
-drop function convslot_trig4();
