@@ -1,3 +1,6 @@
+
+# Copyright (c) 2021, PostgreSQL Global Development Group
+
 # Test logical replication with partitioned tables
 use strict;
 use warnings;
@@ -575,7 +578,6 @@ $result = $node_subscriber2->safe_psql('postgres',
 	"SELECT a FROM tab4_1 ORDER BY 1");
 is( $result, qq(), 'inserts into tab4_1 replicated');
 
-
 # update (replicated as update)
 $node_publisher->safe_psql('postgres', "UPDATE tab1 SET a = 6 WHERE a = 5");
 $node_publisher->safe_psql('postgres', "UPDATE tab2 SET a = 6 WHERE a = 5");
@@ -734,7 +736,8 @@ is($result, qq(), 'truncate of tab3_1 replicated');
 # check that the map to convert tuples from leaf partition to the root
 # table is correctly rebuilt when a new column is added
 $node_publisher->safe_psql('postgres',
-	"ALTER TABLE tab2 DROP b, ADD COLUMN c text DEFAULT 'pub_tab2', ADD b text");
+	"ALTER TABLE tab2 DROP b, ADD COLUMN c text DEFAULT 'pub_tab2', ADD b text"
+);
 $node_publisher->safe_psql('postgres',
 	"INSERT INTO tab2 (a, b) VALUES (1, 'xxx'), (3, 'yyy'), (5, 'zzz')");
 $node_publisher->safe_psql('postgres',
